@@ -61,7 +61,7 @@ function defineCommonExtensionSymbols(apiPrivate)
         AuditStarted: "audit-started-",
         ButtonClicked: "button-clicked-",
         ConsoleMessageAdded: "console-message-added",
-        ElementsPanelObjectSelected: "panel-objectSelected-elements",
+        PanelObjectSelected: "panel-objectSelected-",
         NetworkRequestFinished: "network-request-finished",
         OpenResource: "open-resource",
         PanelSearch: "panel-search-",
@@ -81,6 +81,7 @@ function defineCommonExtensionSymbols(apiPrivate)
         CreateSidebarPane: "createSidebarPane",
         CreateStatusBarButton: "createStatusBarButton",
         EvaluateOnInspectedPage: "evaluateOnInspectedPage",
+        ForwardKeyboardEvent: "_forwardKeyboardEvent",
         GetConsoleMessages: "getConsoleMessages",
         GetHAR: "getHAR",
         GetPageResources: "getPageResources",
@@ -95,6 +96,8 @@ function defineCommonExtensionSymbols(apiPrivate)
         SetSidebarPage: "setSidebarPage",
         ShowPanel: "showPanel",
         StopAuditCategoryRun: "stopAuditCategoryRun",
+        OpenResource: "openResource",
+        Reload: "Reload",
         Unsubscribe: "unsubscribe",
         UpdateAuditProgress: "updateAuditProgress",
         UpdateButton: "updateButton",
@@ -184,16 +187,6 @@ function InspectorExtensionAPI()
     this.console = new ConsoleAPI();
 }
 
-/**
- * @constructor
- */
-InspectorExtensionAPI.prototype = {
-    log: function(message)
-    {
-        extensionServer.sendRequest({ command: commands.Log, message: message });
-    }
-}
-
 /**
  * @constructor
  */
@@ -281,7 +274,8 @@ RequestImpl.prototype = {
 function Panels()
 {
     var panels = {
-        elements: new ElementsPanel()
+        elements: new ElementsPanel(),
+        sources: new SourcesPanel(),
     };
 
     function panelGetter(name)
@@ -330,6 +324,11 @@ Panels.prototype = {
             extensionServer.sendRequest({ command: commands.SetOpenResourceHandler, "handlerPresent": !!callback });
     },
 
+    openResource: function(url, lineNumber, callback)
+    {
+        extensionServer.sendRequest({ command: commands.OpenResource, "url": url, "lineNumber": lineNumber }, callback);
+    },
+
     get SearchAction()
     {
         return apiPrivate.panels.SearchAction;
@@ -346,7 +345,10 @@ function ExtensionViewImpl(id)
     function dispatchShowEvent(message)
     {
         var frameIndex = message.arguments[0];
-        this._fire(window.parent.frames[frameIndex]);
+        if (typeof frameIndex === "number")
+            this._fire(window.parent.frames[frameIndex]);
+        else
+            this._fire();
     }
     this.onShown = new EventSink(events.ViewShown + id, dispatchShowEvent);
     this.onHidden = new EventSink(events.ViewHidden + id);
@@ -355,9 +357,10 @@ function ExtensionViewImpl(id)
 /**
  * @constructor
  */
-function PanelWithSidebarImpl(id)
+function PanelWithSidebarImpl(hostPanelName)
 {
-    this._id = id;
+    this._hostPanelName = hostPanelName;
+    this.onSelectionChanged = new EventSink(events.PanelObjectSelected + hostPanelName);
 }
 
 PanelWithSidebarImpl.prototype = {
@@ -366,7 +369,7 @@ PanelWithSidebarImpl.prototype = {
         var id = "extension-sidebar-" + extensionServer.nextObjectId();
         var request = {
             command: commands.CreateSidebarPane,
-            panel: this._id,
+            panel: this._hostPanelName,
             id: id,
             title: title
         };
@@ -386,9 +389,16 @@ PanelWithSidebarImpl.prototype = {
  */
 function ElementsPanel()
 {
-    var id = "elements";
-    PanelWithSidebar.call(this, id);
-    this.onSelectionChanged = new EventSink(events.ElementsPanelObjectSelected);
+    PanelWithSidebar.call(this, "elements");
+}
+
+/**
+ * @constructor
+ * @extends {PanelWithSidebar}
+ */
+function SourcesPanel()
+{
+    PanelWithSidebar.call(this, "sources");
 }
 
 /**
@@ -728,6 +738,27 @@ function TimelineImpl()
     this.onEventRecorded = new EventSink(events.TimelineEventRecorded);
 }
 
+function forwardKeyboardEvent(event)
+{
+    const Esc = "U+001B";
+    // We only care about global hotkeys, not about random text
+    if (!event.ctrlKey && !event.altKey && !event.metaKey && !/^F\d+$/.test(event.keyIdentifier) && event.keyIdentifier !== Esc)
+        return;
+    var request = {
+        command: commands.ForwardKeyboardEvent,
+        eventType: event.type,
+        ctrlKey: event.ctrlKey,
+        altKey: event.altKey,
+        metaKey: event.metaKey,
+        keyIdentifier: event.keyIdentifier,
+        location: event.location
+    };
+    extensionServer.sendRequest(request);
+}
+
+document.addEventListener("keydown", forwardKeyboardEvent, false);
+document.addEventListener("keypress", forwardKeyboardEvent, false);
+
 /**
  * @constructor
  */
@@ -955,5 +986,6 @@ function platformExtensionAPI(coreAPI)
 
         var tabId;
         var extensionInfo = {};
+        var extensionServer;
         platformExtensionAPI(injectedExtensionAPI("remote-" + window.parent.frames.length));
     })();
\ No newline at end of file
